@@ -7,19 +7,16 @@ use feature ':5.10';
 use Yote::Array;
 use Yote::Hash;
 use Yote::Obj;
+use Yote::YoteRoot;
+use Yote::SQLiteIO;
 
 use WeakRef;
 
-use Exporter;
-use base 'Exporter';
-
-our @EXPORT_OK = qw(fetch stow a_child_of_b);
-
 $Yote::ObjProvider::DIRTY = {};
+$Yote::ObjProvider::PKG_TO_METHODS = {};
 $Yote::ObjProvider::WEAK_REFS = {};
 
 our $DATASTORE;
-our $CACHE;
 
 use vars qw($VERSION);
 
@@ -31,24 +28,23 @@ $VERSION = '0.01';
 
 sub init {
     my $args = ref( $_[0] ) ? $_[0] : { @_ };
-    my $ds = $args->{datastore} || 'Yote::SQLiteIO';
-    eval("use $ds");
-    die $@ if $@;
-    $DATASTORE = $ds->new( $args );
+    $DATASTORE = new Yote::SQLiteIO( $args );
     $DATASTORE->ensure_datastore();
-    my $cache = $args->{cache} || 'Yote::Cache';
-    eval("use $cache");
-    $CACHE = $cache->new( $args );
+    new Yote::YoteRoot(); #ensure that there is the singleton root object.
 } #init
 
-sub connect {
-    return $DATASTORE->connect();
-}
-
 sub disconnect {
     return $DATASTORE->disconnect();
 }
 
+sub start_transaction {
+    return $DATASTORE->start_transaction();
+}
+
+sub commit_transaction {
+    return $DATASTORE->commit_transaction();
+}
+
 sub xpath {
     my $path = shift;
     return xform_out( $DATASTORE->xpath( $path ) );
@@ -60,11 +56,7 @@ sub xpath_count {
 }
 
 #
-# Deep clone this object. This will clone any yote object that is not an AppRoot or flagged
-# to not deep clone.
-# If an object is referenced below itself, the references below it will be set to the 
-# reference of the clone. For example, if you have the structure $A = { "foo" =>  $B(NO_DEEP_CLONE), "bar" => [1,2,$A,$C] } 
-# and you deep clone A, you will get $Aclone = { "foo" => $B, "bar" => [1,2,$Aclone,$Cclone] }
+# Deep clone this object. This will clone any yote object that is not an AppRoot.
 #
 sub power_clone {
     my( $item, $replacements ) = @_;
@@ -74,7 +66,7 @@ sub power_clone {
     my $at_start = 0;
     unless( $replacements ) {
         $at_start = 1;
-        $replacements = {};
+        $replacements ||= {};
     }
     my $id = get_id( $item );
     return $replacements->{$id} if $replacements->{$id};
@@ -92,11 +84,12 @@ sub power_clone {
         return $hash_clone;
     }
     else {
-        return $item if $item->{NO_DEEP_CLONE} && (! $at_start);
+	print STDERR Data::Dumper->Dump(["BOINK", $item]) if $item->isa( 'Yote::AppRoot' ) && (! $at_start);
+        return $item if $item->isa( 'Yote::AppRoot' ) && (! $at_start);
     }
 
     my $clone = $class->new;
-    $replacements->{$item->{ID}} = get_id( $clone );
+    $replacements->{ $id } = get_id( $clone );
 
     for my $field (keys %{$item->{DATA}}) {
         my $id_or_val = $item->{DATA}{$field};
@@ -106,6 +99,17 @@ sub power_clone {
             $clone->{DATA}{$field} = $id_or_val;
         }
     }
+
+    if( $at_start ) {
+	my( @cloned ) = map { Yote::ObjProvider::fetch($_)  } keys %$replacements;
+	my( %cloned );
+	for my $obj (@cloned) {
+	    $cloned{ ref( $obj ) }++;
+	}
+	print STDERR Data::Dumper->Dump(["Cloned " .scalar(keys %$replacements)." items"]);
+	print STDERR Data::Dumper->Dump([\%cloned]);
+    }
+
     return $clone;
     
 } #power_clone
@@ -121,7 +125,7 @@ sub fetch {
     #
     # Return the object if we have a reference to its dirty state.
     #
-    my $ref = $CACHE->fetch($id_or_xpath) || $Yote::ObjProvider::DIRTY->{$id_or_xpath} || $Yote::ObjProvider::WEAK_REFS->{$id_or_xpath};
+    my $ref = $Yote::ObjProvider::DIRTY->{$id_or_xpath} || $Yote::ObjProvider::WEAK_REFS->{$id_or_xpath};
     return $ref if $ref;
 
     my $obj_arry = $DATASTORE->fetch( $id_or_xpath );
@@ -133,14 +137,14 @@ sub fetch {
                 my( @arry );
                 tie @arry, 'Yote::Array', $id_or_xpath, @$data;
                 my $tied = tied @arry; $tied->[2] = \@arry;
-                store_weak( $id_or_xpath, \@arry );
+                _store_weak( $id_or_xpath, \@arry );
                 return \@arry;
             }
             when('HASH') {
                 my( %hash );
                 tie %hash, 'Yote::Hash', $id_or_xpath, map { $_ => $data->{$_} } keys %$data;
                 my $tied = tied %hash; $tied->[2] = \%hash;
-                store_weak( $id_or_xpath, \%hash );
+                _store_weak( $id_or_xpath, \%hash );
                 return \%hash;
             }
             default {
@@ -148,7 +152,7 @@ sub fetch {
                 my $obj = $class->new( $id_or_xpath );
                 $obj->{DATA} = $data;
                 $obj->{ID} = $id_or_xpath;
-                store_weak( $id_or_xpath, $obj );
+                _store_weak( $id_or_xpath, $obj );
                 return $obj;
             }
         }
@@ -156,25 +160,6 @@ sub fetch {
     return undef;
 } #fetch
 
-sub row_size { #returns number of rows this object has.
-    my $ref = shift;
-    if( ref($ref) eq 'ARRAY' ) {
-        return scalar @$ref;
-    }
-    elsif( ref($ref) eq 'HASH' ) {
-        return scalar keys %$ref;
-    }
-    elsif( ref($ref) eq 'Yote::Array' ) {
-        return $ref->FETCHSIZE();
-    }
-    elsif( ref($ref) eq 'Yote::Hash' ) {
-        return $ref->keycount();
-    }
-    else {
-        return $ref->size();
-    }
-} #row_size
-
 sub get_id {
     my $ref = shift;
     my $class = ref( $ref );
@@ -186,7 +171,7 @@ sub get_id {
             my $tied = tied @$ref;
             if( $tied ) {
                 $tied->[0] ||= $DATASTORE->get_id( "ARRAY" );
-                store_weak( $tied->[0], $ref );
+                _store_weak( $tied->[0], $ref );
                 return $tied->[0];
             }
             my( @data ) = @$ref;
@@ -195,7 +180,7 @@ sub get_id {
             my $tied = tied @$ref; $tied->[2] = $ref;
             push( @$ref, @data );
             dirty( $ref, $id );
-            store_weak( $id, $ref );
+            _store_weak( $id, $ref );
             return $id;
         }
         when('Yote::Hash') {
@@ -207,7 +192,7 @@ sub get_id {
 
             if( $tied ) {
                 $tied->[0] ||= $DATASTORE->get_id( "HASH" );
-                store_weak( $tied->[0], $ref );
+                _store_weak( $tied->[0], $ref );
                 return $tied->[0];
             }
             my $id = $DATASTORE->get_id( $class );
@@ -218,17 +203,37 @@ sub get_id {
                 $ref->{$key} = $vals{$key};
             }
             dirty( $ref, $id );
-            store_weak( $id, $ref );
+            _store_weak( $id, $ref );
             return $id;
         }
         default {
             $ref->{ID} ||= $DATASTORE->get_id( $class );
-            store_weak( $ref->{ID}, $ref );
+            _store_weak( $ref->{ID}, $ref );
             return $ref->{ID};
         }
     }
 } #get_id
 
+sub package_methods {
+    my $pkg = shift;
+    my $methods = $Yote::ObjProvider::PKG_TO_METHODS{$pkg};
+    unless( $methods ) {
+
+        no strict 'refs';
+
+        my @m = grep { $_ && $_ !~ /^(_.*|AUTOLOAD|BEGIN|DESTROY|CLONE_SKIP|ISA|VERSION|add_to_.*|remove_from_.*|init|import|set_.*|get_.*|can|isa|new)$/ } keys %{"${pkg}\::"};
+
+        for my $class ( @{"${pkg}\::ISA" } ) {
+            my $pm = package_methods( $class );
+            push @m, @$pm;
+        }
+        $methods = \@m;
+        $Yote::ObjProvider::PKG_TO_METHODS{$pkg} = $methods;
+        use strict 'refs';
+    }
+    return $methods;
+} #package_methods
+
 sub a_child_of_b {
     my( $a, $b, $seen ) = @_;
     my $bref = ref( $b );
@@ -265,16 +270,102 @@ sub stow_all {
     }
 } #stow_all
 
-sub reset {
-    stow_all();
-    $Yote::ObjProvider::DIRTY = {};
-    $Yote::ObjProvider::WEAK_REFS = {};
+sub stow {
+
+    my( $obj ) = @_;
+    my $class = ref( $obj );
+    return unless $class;
+    my $id = get_id( $obj );
+    die unless $id;
+    my $data = _raw_data( $obj );
+    given( $class ) {
+        when('ARRAY') {
+            $DATASTORE->stow( $id,'ARRAY', $data );
+            _clean( $id );
+        }
+        when('HASH') {
+            $DATASTORE->stow( $id,'HASH',$data );
+            _clean( $id );
+        }
+        when('Yote::Array') {
+            if( _is_dirty( $id ) ) {
+                $DATASTORE->stow( $id,'ARRAY',$data );
+                _clean( $id );
+            }
+            for my $child (@$data) {
+                if( $child > 0 && $Yote::ObjProvider::DIRTY->{$child} ) {
+                    stow( $Yote::ObjProvider::DIRTY->{$child} );
+                }
+            }
+        }
+        when('Yote::Hash') {
+            if( _is_dirty( $id ) ) {
+                $DATASTORE->stow( $id, 'HASH', $data );
+            }
+            _clean( $id );
+            for my $child (values %$data) {
+                if( $child > 0 && $Yote::ObjProvider::DIRTY->{$child} ) {
+                    stow( $Yote::ObjProvider::DIRTY->{$child} );
+                }
+            }
+        }
+        default {
+            if( _is_dirty( $id ) ) {
+                $DATASTORE->stow( $id, $class, $data );
+                _clean( $id );
+            }
+            for my $val (values %$data) {
+                if( $val > 0 && $Yote::ObjProvider::DIRTY->{$val} ) {
+                    stow( $Yote::ObjProvider::DIRTY->{$val} );
+                }
+            }
+        }
+    } #given
+
+} #stow
+
+sub xform_out {
+    my $val = shift;
+    return undef unless defined( $val );
+    if( index($val,'v') == 0 ) {
+        return substr( $val, 1 );
+    }
+    return fetch( $val );
 }
 
+sub xform_in {
+    my $val = shift;
+    if( ref( $val ) ) {
+        return get_id( $val );
+    }
+    return "v$val";
+}
+
+
+#
+# Markes given object as dirty.
+#
+sub dirty {
+    my $obj = shift;
+    my $id = shift;
+    $Yote::ObjProvider::DIRTY->{$id} = $obj;
+}
+
+#
+# Returns ids that are dirty
+#
+sub dirty_ids {
+    return keys %$Yote::ObjProvider::DIRTY;
+}
+
+#
+# 'private' methods ----------------------
+#
+
 #
 # Returns data structure representing object. References are integers. Values start with 'v'.
 #
-sub raw_data {
+sub _raw_data {
     my( $obj ) = @_;
     my $class = ref( $obj );
     return unless $class;
@@ -307,108 +398,26 @@ sub raw_data {
             return $obj->{DATA};
         }
     }
-} #raw_data
+} #_raw_data
 
-sub stow {
-
-    my( $obj ) = @_;
-    my $class = ref( $obj );
-    return unless $class;
-    my $id = get_id( $obj );
-    die unless $id;
-    my $data = raw_data( $obj );
-    given( $class ) {
-        when('ARRAY') {
-            $DATASTORE->stow( $id,'ARRAY', $data );
-            clean( $id );
-        }
-        when('HASH') {
-            $DATASTORE->stow( $id,'HASH',$data );
-            clean( $id );
-        }
-        when('Yote::Array') {
-            if( is_dirty( $id ) ) {
-                $DATASTORE->stow( $id,'ARRAY',$data );
-                clean( $id );
-            }
-            for my $child (@$data) {
-                if( $child > 0 && $Yote::ObjProvider::DIRTY->{$child} ) {
-                    stow( $Yote::ObjProvider::DIRTY->{$child} );
-                }
-            }
-        }
-        when('Yote::Hash') {
-            if( is_dirty( $id ) ) {
-                $DATASTORE->stow( $id, 'HASH', $data );
-            }
-            clean( $id );
-            for my $child (values %$data) {
-                if( $child > 0 && $Yote::ObjProvider::DIRTY->{$child} ) {
-                    stow( $Yote::ObjProvider::DIRTY->{$child} );
-                }
-            }
-        }
-        default {
-            if( is_dirty( $id ) ) {
-                $DATASTORE->stow( $id, $class, $data );
-                clean( $id );
-            }
-            for my $val (values %$data) {
-                if( $val > 0 && $Yote::ObjProvider::DIRTY->{$val} ) {
-                    stow( $Yote::ObjProvider::DIRTY->{$val} );
-                }
-            }
-        }
-    } #given
-
-} #stow
-
-sub xform_out {
-    my $val = shift;
-    return undef unless defined( $val );
-    if( index($val,'v') == 0 ) {
-        return substr( $val, 1 );
-    }
-    return fetch( $val );
-}
-
-sub xform_in {
-    my $val = shift;
-    if( ref( $val ) ) {
-        return get_id( $val );
-    }
-    return "v$val";
-}
-
-sub store_weak {
+sub _store_weak {
     my( $id, $ref ) = @_;
     die "SW" if ref($ref) eq 'Yote::Hash';
-    $CACHE->stow( $id, $ref );
     my $weak = $ref;
     weaken( $weak );
     $Yote::ObjProvider::WEAK_REFS->{$id} = $weak;
-}
+} #_store_weak
 
-sub dirty {
-    my $obj = shift;
-    my $id = shift;
-    $Yote::ObjProvider::DIRTY->{$id} = $obj;
-}
-
-sub dirty_ids {
-    return keys %$Yote::ObjProvider::DIRTY;
-}
-
-sub is_dirty {
+sub _is_dirty {
     my $obj = shift;
     my $id = ref($obj) ? get_id($obj) : $obj;
     return $Yote::ObjProvider::DIRTY->{$id};
-}
+} #_is_dirty
 
-sub clean {
+sub _clean {
     my $id = shift;
     delete $Yote::ObjProvider::DIRTY->{$id};
-}
+} #_clean
 
 1;
 __END__
